package org.greenstone.gsdl3.gs3build.util;

import java.io.File;
import java.io.FileReader;

import java.net.URL;

import org.w3c.dom.*;

import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.ext.LexicalHandler;

import org.greenstone.gsdl3.gs3build.doctypes.DocumentID;
import org.greenstone.gsdl3.gs3build.doctypes.DocumentInterface;
import org.greenstone.gsdl3.gs3build.doctypes.DocumentList;
import org.greenstone.gsdl3.gs3build.doctypes.GMLDocument;

public class HTMLTidy extends DefaultHandler
{
  String label;
  File   file;
  StringBuffer value;
  boolean inElement;
  boolean accumulate;
  Document document;
  Element  currentElement;
  Element root;

  class HTMLCommentHandler implements LexicalHandler
  {
    HTMLTidy parent;

    HTMLCommentHandler(HTMLTidy tidier)
    { this.parent = tidier;
    }

    public void comment(char ch[], int start, int length)
    { String comment = new String(ch, start, length);

      parent.comment(comment);
    }

    public void endCDATA()
    {
    }

    public void endDTD()
    {
    }

    public void endEntity(String name)
    {
    }

    public void startCDATA()
    {
    }

    public void startDTD(String name, String publicId, String systemId)
    {
    }

    public void startEntity(String name)
    {
    }
  }

  public HTMLTidy(URL url)
  { // TODO: complete
  }

  public HTMLTidy(File file)
  { this.file = null;
    this.label = null;
    this.value = null;

    this.document = new org.apache.html.dom.HTMLDocumentImpl();

    this.currentElement = this.document.getDocumentElement();
    this.root = this.currentElement;

    try {
      XMLReader handler  = new org.ccil.cowan.tagsoup.Parser();
      HTMLCommentHandler commentHandler = new HTMLCommentHandler(this);
      handler.setContentHandler(this);
      handler.setProperty("http://xml.org/sax/properties/lexical-handler", commentHandler);
      handler.parse(file.toString());
    }
    catch (java.io.IOException io)
    {
    }
    catch (org.xml.sax.SAXException saxEx)
    {
      System.out.println(saxEx);
    }
  }

  public void startElement(String URI, String localName, String qName, Attributes attributes)
  { //System.out.println("<"+localName+">");
  
    if (localName.equals("html") == false) {
      Element childElement = this.document.createElement(localName);
      
      this.currentElement.appendChild(childElement);
      this.currentElement = childElement;

      for (int a = 0; a < attributes.getLength(); a ++)
      { String name = attributes.getLocalName(a);
        String value = attributes.getValue(a);

	childElement.setAttribute(name, value);
      }
    }
  }

  public void comment(String text)
  {
    Comment comment = document.createComment(text);
    this.currentElement.appendChild(comment);
    //System.out.println("<!-- "+text+" -->");
  }

  public void endElement(String URI, String localName, String qName)
  { // System.out.println("Ending " + localName);
    // System.out.println("</"+localName+">");

    if (localName.equals("html")) {
      return;
    }

    // TODO: check is the same name as current node
    if (!this.currentElement.getTagName().equals(localName.toUpperCase())) {
      // TODO: error handling
      System.out.println("Bad end at " + localName + " " + this.currentElement.getTagName());
      return;
    }

    // Optimisation: is this an empty tag?
    boolean isEmpty = false;

    if (this.currentElement.getChildNodes().getLength() == 1)
    { Node child = this.currentElement.getFirstChild();

      if (child.getNodeType() == Node.TEXT_NODE) {
	String text = child.getNodeValue();
	
	int i = 0;
	while (i < text.length())
	{ if (text.charAt(i) > 32 && text.charAt(i) != 160) {
	    break;
	  }
	  i++;
	}
	if (text.length() == i) {
	  isEmpty = true;
	}
      }
    }
    else if (this.currentElement.getChildNodes().getLength() == 0)
    { isEmpty = true;
    }

    boolean isSingleton = false;
    String tagName = this.currentElement.getTagName().toLowerCase();

    if (tagName.equals("img") ||
	tagName.equals("br") ||
	tagName.equals("meta") ||
	tagName.equals("input") ||
	tagName.equals("area") ||
	tagName.equals("link") ||
	tagName.equals("base") ||
	tagName.equals("img") ||
	tagName.equals("hr")) {
      isSingleton = true;
    }

    Element parent =  (Element) this.currentElement.getParentNode();

    if (isEmpty == true && !isSingleton) {
      // trim the current child
      parent.removeChild(this.currentElement);
      //      System.out.println("***Removing child***");
    }

    this.currentElement = parent;
  }

  public void characters(char c[], int start, int length)
  { String string = new String(c, start, length);
    Node text_node = this.document.createTextNode(string);
    this.currentElement.appendChild(text_node);
    //    System.out.println(string);
  }

  public void recursePrint(Element element, int indent)
  {
    for (int i = 0; i < indent; i++) {
      System.out.print(" ");
    }
    System.out.println(element.toString());
    for (int c = 0; c < element.getChildNodes().getLength(); c ++)
    { Node n = element.getChildNodes().item(c);

      if (n.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE) {
	Element e = (Element) n;
	
	this.recursePrint(e, indent+2);
      }
    }
  }

  public Document getDocument()
  { 
    //    this.recursePrint(this.root, 0);
    //    System.out.println(this.document.toString());
    //    System.out.println(this.root);
    return this.document;
  }
}
